﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using MaxIm;

namespace MxPlugin_CombineDocs
{

    // COM Required Attributes for a custom coclass Interface
    // You must replace this GUID with a custom value.
    // Go to "Tools" > "Create GUID" to create your own.
    [Guid("1E277648-F4B0-40EA-B918-ED3F4A97E7E2")]
    [CoClass(typeof(PlugIn))]
    [ComImport]
    [InterfaceType(ComInterfaceType.InterfaceIsDual)]
    public interface _PlugIn
    {
        [DispId(0)] void DoModal([In, MarshalAs(UnmanagedType.Interface)] MaxIm.Document doc);

        [DispId(1)]
        [return: MarshalAs(UnmanagedType.BStr)]
        string Name();
    }

    // The coclass implementation of the COM Interface.
    // You must replace this GUID with a custom value.
    // Go to "Tools" > "Create GUID" to create your own.
    [Guid("E57DD169-B605-4A09-AC4F-C5B67CEDB65D")]
    [ClassInterface(ClassInterfaceType.AutoDual)]
    public class PlugIn : _PlugIn
    {
        /*!
         * \brief Class constructor
         * Default class contructor must be implemented for COM objects.
         */
        public PlugIn()
        {
            // Initialize the COM object.
        }

        /*!
         * \brief Function executed on PlugIn execute call.
         * DoModal is the function call passed through the COM to the PlugIn. Takes the currently open MaxIm DL document as an argument. 
         * \param MaxIm.Document doc Document to execute the plugin on. Provides direct access to the Image Buffer SAFEARRAY, rather than through a VARIANT array. 
         *        Is the open document with current focus in MaxIm DL.
         */
        public void DoModal(MaxIm.Document doc)
        {
            // Do stuff here.

            MaxIm.Application myApp = new MaxIm.Application();
            MaxIm.Documents myDocs = myApp.Documents;
            Microsoft.VisualBasic.Collection docList = new Microsoft.VisualBasic.Collection();


            int docsCount = myDocs.Count();

            MaxIm.Document doc2 = null;

            for (int i = 1; i <= docsCount; i++)
            {
                doc2 = (MaxIm.Document)(object)(myDocs.Item(i));
                docList.Add(doc2);
            }


            MaxIm.Document targetDoc = new MaxIm.Document();
            targetDoc.CombineImages(MaxIm.AlignType.mxNoAlignment, true, MaxIm.CombineType.mxMedianCombine, false, docList);
            targetDoc.Modified = true;

            Marshal.ReleaseComObject(targetDoc);
            Marshal.ReleaseComObject(doc2);
            Marshal.ReleaseComObject(myDocs);
            Marshal.ReleaseComObject(myApp);

        }

        /*!
         * \brief Returns the name of the PlugIn for use in the MaxIm DL GUI.
         * Returns the name of the PlugIn for use in the MaxIm DL GUI.
         */
        public string Name()
        {
            return "MaxIm DL Sample C# Plugin - CombineDocs";
        }
    }

}

